code		segment
		assume	cs:code,ds:code
		org	100h
		.286
start:		mov	ax,0003h	; video mode (80x25 16 colors)
		int	10h


		mov	si,80h		; preparing command line
		xor	bx,bx
		mov	bl,81h
		add	bl,[si]
		mov	byte ptr [bx],0

		mov	dx,81h		; opening file
		mov	ax,3D00h
		int	21h
		jnc	open_ok
		jmp	abort
open_ok:	mov	[handle],ax

from_top:	xor	ax,ax
		mov	[cur_ofs],ax
		mov	[cur_ofs+2],ax
main_cycle:	lea	si,cur_ofs
		lea	di,top_ofs
		mov	cx,2
		rep	movsw
		call	draw_screen
check_kb:	lea	si,top_ofs
		lea	di,cur_ofs
		mov	bx,di
		mov	cx,2
		rep	movsw

		mov	ah,10h
		int	16h
		mov	cx,1
		cmp	ah,72		; UP
		jz	up
		cmp	ah,80		; DOWN
		jz	down
		mov	cx,25
		cmp	ah,73		; PAGEUP
		jz	up
		cmp	al,'b'		; 'b'
		jz	up
		cmp	ah,81		; PAGEDOWN
		jz	down
		mov	cx,65535
		cmp	ah,71		; HOME
		jz	up
		cmp	ah,79		; END
		jz	down
		cmp	al,' '		; SPACE
		jz	down
		cmp	ah,1		; ESC
		jz	exit
		jmp	check_kb

up:		call	ldec
		jc	from_top
		call	ldec
		jc	from_top
		call	ldec
		jc	from_top

pc:		call	get_char
		call	ldec
		jc	from_top
		
		mov	al,[char]
		call	get_char
		mov	ah,[char]
		cmp	ax,0D0Ah	; CR/LF
		jne	pc
		call	linc
		call	linc
		loop	up
		jmp	main_cycle
down:		call	get_char
		jc	check_kb
		mov	al,[char]
		call	linc
		call	get_char
		jc	cont_kb
		mov	ah,[char]
		cmp	ax,0A0Dh	; CR/LF
		jne	down
		call	linc
		loop	down
		jmp	main_cycle

cont_kb:	jmp	check_kb

exit:		mov	bx,[handle]	; closing file
		mov	ah,3Eh
		int	21h
		mov	ax,0003h
		int	10h
		int	20h

draw_screen	proc
		cld
		pusha
		push	es

		push	ds
		pop	es
		lea	si,top_ofs
		lea	di,cur_ofs
		mov	cx,2
		rep	movsw

		mov	ax,0B800h	; drawing from current position
		mov	es,ax
		xor	di,di
		mov	cx,25
next_line:	xor	dx,dx
		lea	bx,cur_ofs
next_char:	call	get_char
		jc	eol
		call	linc
		mov	al,[char]
		call	get_char
		mov	ah,[char]
		jc	put_char
		cmp	ax,0A0Dh
		jne	put_char
		call	linc
		jmp	eol
put_char:	cmp	dx,80
		jae	next_char		
		mov	ah,17h
		stosw
		inc	dx
		jmp	next_char
eol:		cmp	dx,80
		jae	skip_clearing
		push	cx
		mov	cx,80
		sub	cx,dx
		cld
		mov	ax,1720h
		rep	stosw
		pop	cx
skip_clearing:	loop	next_line
		pop	es
		popa
		ret
draw_screen	endp

abort:		mov	ax,0003h
		int	10h
		lea	dx,abort_msg
		mov	ah,9
		int	21h
		int	20h

get_char	proc	; result goes in al
		pusha
		mov	ax,[cur_ofs]
		mov	dx,[cur_ofs + 2]

		mov	bx,[buf_len]
		div	bx

		cmp	ax,[cur_page]
		push	dx
		
		je	loaded

		mov	[cur_page],ax

		mul	bx

		mov	cx,dx
		mov	dx,ax
		mov	ax,4200h
		mov	bx,[handle]
		int	21h
		jc	abort
		mov	cx,[buf_len]
		lea	dx,buf
		mov	ah,3Fh
		int	21h
		jc	abort
		mov	[buf_fill],ax
loaded:		pop	si
		mov	al,[si+offset buf]
		cmp	[buf_fill],si
		ja	get_byte_exit
		stc
get_byte_exit:	mov	[char],al
		popa
		ret
get_char	endp

ldec		proc
		push	cx
		mov	cx,1
		sub	[bx],cx
		jnc	no_underflow
		sub	word ptr [bx+2],cx
no_underflow:
		pop	cx
		ret
ldec		endp

linc		proc
		push	cx
		mov	cx,1
		add	[bx],cx
		jnc	no_overflow
		inc	word ptr [bx+2]
no_overflow:
		pop	cx
		ret
linc		endp

abort_msg	db	'Some error occured!',0dh,0ah,'$'

handle		dw	0

char		db	0

top_ofs		dw	0,0
cur_ofs		dw	0,0

cur_page	dw	0FFFFh
buf_len		dw	8000h
buf_fill	dw	0h

buf:

code		ends
		end	start